<?php

namespace JsonApi\Routes\Blubber;

use BlubberComment;
use BlubberThread;
use Course;
use User;

class Authority
{
    public static function canShowBlubberThread(User $user, BlubberThread $resource)
    {
        return self::userIsAuthor($user) && $resource->isReadable($user->id);
    }

    public static function canEditBlubberThread(User $user, BlubberThread $resource): bool
    {
        return self::canShowBlubberThread($user, $resource);
    }

    public static function canCreatePrivateBlubberThread(User $user)
    {
        return self::userIsAuthor($user);
    }

    public static function canCreateCourseBlubberThread(User $user, Course $course)
    {
        return self::userIsTeacher($user, $course);
    }

    public static function canEditCourseBlubberThread(User $user, Course $course)
    {
        return self::userIsTeacher($user, $course);
    }

    public static function canCreateComment(User $user, BlubberThread $resource)
    {
        return self::userIsAuthor($user) && $resource->isCommentable($user->id);
    }

    public static function canDeleteComment(User $user, BlubberComment $resource)
    {
        return self::canEditComment($user, $resource);
    }

    public static function canEditComment(User $user, BlubberComment $resource)
    {
        return self::userIsAuthor($user) && $resource->isWritable($user->id);
    }

    public static function canIndexComments(User $user, BlubberThread $resource = null)
    {
        return isset($resource)
            ? self::canShowBlubberThread($user, $resource)
            : self::userIsAuthor($user);
    }

    public static function canShowComment(User $user, BlubberComment $resource)
    {
        return self::canShowBlubberThread($user, $resource->thread);
    }

    /**
     * @SuppressWarnings(PHPMD.Superglobals)
     */
    private static function userIsAuthor(User $user)
    {
        return $GLOBALS['perm']->have_perm('autor', $user->id);
    }

    /**
     * @SuppressWarnings(PHPMD.Superglobals)
     */
    private static function userIsTeacher(User $user, Course $course)
    {
        return $GLOBALS['perm']->have_studip_perm('tutor', $course->id, $user->id);
    }
}
